#========================================================================================
# package      DE_EPAGES::Shop::t::unit::ShopUriBuilder
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for ShopUriBuilder.pm
#========================================================================================
package DE_EPAGES::Shop::t::unit::ShopUriBuilder;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Shop::API::ShopUriBuilder;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->test_withoutParams();
    $self->test_withTrailingSlash();
    $self->test_withoutTrailingSlash();
    $self->test_withProxyAndWithoutTrailingSlash();
    $self->test_withProxyAndTrailingSlash();
    $self->test_withMultipleTrailingSlash();
    $self->test_withoutLeadingPathSlash();
    $self->test_withMultipleLeadingPathSlashes();
    $self->test_withQueryParams();
    $self->test_withoutQueryParams();
    $self->test_withBrokenQueryParams();

    done_testing();
    return;
}

sub test_withoutParams {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->_buildShop({
        'RESTAPIURL' => 'https://shop-domain.tld/',
    });
    my $PathTemplate = '/rs/some/resource';
    my $hPathParams = {};
    my $System = $self->_buildSystem({
        'EPagesJBaseURL' => 'http://unstable-main.epages.com:8088/rs/'
    });

    my $SUT = DE_EPAGES::Shop::API::ShopUriBuilder->new($Shop, $System);
    my $PublicUri = $SUT->buildUri($PathTemplate, $hPathParams);
    my $PrivateUri = $SUT->buildPrivateUri($PathTemplate, $hPathParams);

    is($PublicUri, 'https://shop-domain.tld/rs/some/resource', "$SubTestName - (PublicUri) without path-params.");
    is($PrivateUri, 'http://unstable-main.epages.com:8088/rs/rs/some/resource', "$SubTestName - ($PrivateUri) without path-params.");
}

sub test_withTrailingSlash {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->_buildShop({
        'RESTAPIURL' => 'https://shop-domain.tld/',
    });
    my $PathTemplate = '/rs/{shopAlias}/some/resource/{recordId}';
    my $hPathParams = {
        'shopAlias' => 'DemoShop',
        'recordId' => '123',
    };
    my $System = $self->_buildSystem({
        'EPagesJBaseURL' => 'http://unstable-main.epages.com:8088/rs/'
    });

    my $SUT = DE_EPAGES::Shop::API::ShopUriBuilder->new($Shop, $System);
    my $PublicUri = $SUT->buildUri($PathTemplate, $hPathParams);
    my $PrivateUri = $SUT->buildPrivateUri($PathTemplate, $hPathParams);

    is($PublicUri, 'https://shop-domain.tld/rs/DemoShop/some/resource/123', "$SubTestName - ($PublicUri) with trailing slash.");
    is($PrivateUri, 'http://unstable-main.epages.com:8088/rs/rs/DemoShop/some/resource/123', "$SubTestName - ($PrivateUri) with trailing slash.");
}

sub test_withoutTrailingSlash {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->_buildShop({
        'RESTAPIURL' => 'https://shop-domain.tld',
    });
    my $PathTemplate = '/rs/{shopAlias}/some/resource/{recordId}';
    my $hPathParams = {
        'shopAlias' => 'DemoShop',
        'recordId' => '123',
    };
    my $System = $self->_buildSystem({
        'EPagesJBaseURL' => 'http://unstable-main.epages.com:8088/rs/'
    });

    my $SUT = DE_EPAGES::Shop::API::ShopUriBuilder->new($Shop, $System);
    my $PublicUri = $SUT->buildUri($PathTemplate, $hPathParams);
    my $PrivateUri = $SUT->buildPrivateUri($PathTemplate, $hPathParams);

    is($PublicUri, 'https://shop-domain.tld/rs/DemoShop/some/resource/123', "$SubTestName - ($PublicUri) without trailing slash.");
    is($PrivateUri, 'http://unstable-main.epages.com:8088/rs/rs/DemoShop/some/resource/123', "$SubTestName - ($PrivateUri) without trailing slash.");
}

sub test_withProxyAndWithoutTrailingSlash {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->_buildShop({
        'RESTAPIURL' => 'https://proxy-domain.tld/shop-domain.tld/epages',
    });
    my $PathTemplate = '{shopAlias}/some/resource/{recordId}';
    my $hPathParams = {
        'shopAlias' => 'DemoShop',
        'recordId' => '123',
    };
    my $System = $self->_buildSystem({
        'EPagesJBaseURL' => 'http://unstable-main.epages.com:8088/rs'
    });

    my $SUT = DE_EPAGES::Shop::API::ShopUriBuilder->new($Shop, $System);
    my $PublicUri = $SUT->buildUri($PathTemplate, $hPathParams);
    my $PrivateUri = $SUT->buildPrivateUri($PathTemplate, $hPathParams);

    is($PublicUri, 'https://proxy-domain.tld/shop-domain.tld/epages/DemoShop/some/resource/123', "$SubTestName - ($PublicUri) proxy in baseUrl without trailing slash.");
    is($PrivateUri, 'http://unstable-main.epages.com:8088/rs/DemoShop/some/resource/123', "$SubTestName - ($PrivateUri) proxy in baseUrl without trailing slash.");
}

sub test_withProxyAndTrailingSlash {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->_buildShop({
        'RESTAPIURL' => 'https://proxy-domain.tld/shop-domain.tld/epages/',
    });
    my $PathTemplate = '/{shopAlias}/some/resource/{recordId}';
    my $hPathParams = {
        'shopAlias' => 'DemoShop',
        'recordId' => '123',
    };
    my $System = $self->_buildSystem({
        'EPagesJBaseURL' => 'http://unstable-main.epages.com:8088/rs/'
    });

    my $SUT = DE_EPAGES::Shop::API::ShopUriBuilder->new($Shop, $System);
    my $PublicUri = $SUT->buildUri($PathTemplate, $hPathParams);
    my $PrivateUri = $SUT->buildPrivateUri($PathTemplate, $hPathParams);

    is($PublicUri, 'https://proxy-domain.tld/shop-domain.tld/epages/DemoShop/some/resource/123', "$SubTestName - ($PublicUri) proxy in baseUrl with trailing slash.");
    is($PrivateUri, 'http://unstable-main.epages.com:8088/rs/DemoShop/some/resource/123', "$SubTestName - ($PrivateUri) proxy in baseUrl with trailing slash.");
}

sub test_withMultipleTrailingSlash {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->_buildShop({
        'RESTAPIURL' => 'https://shop-domain.tld/////',
    });
    my $PathTemplate = '/rs/{shopAlias}/some/resource/{recordId}';
    my $hPathParams = {
        'shopAlias' => 'DemoShop',
        'recordId' => '123',
    };
    my $System = $self->_buildSystem({
        'EPagesJBaseURL' => 'http://unstable-main.epages.com:8088/rs//////'
    });

    my $SUT = DE_EPAGES::Shop::API::ShopUriBuilder->new($Shop, $System);
    my $PublicUri = $SUT->buildUri($PathTemplate, $hPathParams);
    my $PrivateUri = $SUT->buildPrivateUri($PathTemplate, $hPathParams);

    is($PublicUri, 'https://shop-domain.tld/rs/DemoShop/some/resource/123', "$SubTestName - ($PublicUri) with multiple trailing slash.");
    is($PrivateUri, 'http://unstable-main.epages.com:8088/rs/rs/DemoShop/some/resource/123', "$SubTestName - ($PrivateUri) with multiple trailing slash.");
}

sub test_withoutLeadingPathSlash {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->_buildShop({
        'RESTAPIURL' => 'https://shop-domain.tld/',
    });
    my $PathTemplate = 'rs/{shopAlias}/some/resource/{recordId}';
    my $hPathParams = {
        'shopAlias' => 'DemoShop',
        'recordId' => '123',
    };
    my $System = $self->_buildSystem({
        'EPagesJBaseURL' => 'http://unstable-main.epages.com:8088/rs/'
    });

    my $SUT = DE_EPAGES::Shop::API::ShopUriBuilder->new($Shop, $System);
    my $PublicUri = $SUT->buildUri($PathTemplate, $hPathParams);
    my $PrivateUri = $SUT->buildPrivateUri($PathTemplate, $hPathParams);

    is($PublicUri, 'https://shop-domain.tld/rs/DemoShop/some/resource/123', "$SubTestName - ($PublicUri) without a leading slash in path template.");
    is($PrivateUri, 'http://unstable-main.epages.com:8088/rs/rs/DemoShop/some/resource/123', "$SubTestName - ($PrivateUri) without a leading slash in path template.");
}

sub test_withMultipleLeadingPathSlashes {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->_buildShop({
        'RESTAPIURL' => 'https://shop-domain.tld/',
    });
    my $PathTemplate = '////rs/{shopAlias}/some/resource/{recordId}';
    my $hPathParams = {
        'shopAlias' => 'DemoShop',
        'recordId' => '123',
    };
    my $System = $self->_buildSystem({
        'EPagesJBaseURL' => 'http://unstable-main.epages.com:8088/rs/'
    });

    my $SUT = DE_EPAGES::Shop::API::ShopUriBuilder->new($Shop, $System);
    my $PublicUri = $SUT->buildUri($PathTemplate, $hPathParams);
    my $PrivateUri = $SUT->buildPrivateUri($PathTemplate, $hPathParams);

    is($PublicUri, 'https://shop-domain.tld/rs/DemoShop/some/resource/123', "$SubTestName - ($PublicUri) with multiple leading slashes in path template.");
    is($PrivateUri, 'http://unstable-main.epages.com:8088/rs/rs/DemoShop/some/resource/123', "$SubTestName - ($PrivateUri) with multiple leading slashes in path template.");
}

sub test_withQueryParams {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->_buildShop({
        'RESTAPIURL' => 'https://shop-domain.tld/',
    });
    my $PathTemplate = '/rs/some/resource';
    my $hPathParams = {};
    my $hQueyParams = {'search' => 'ball', 'color' => 'ocean'};
    my $System = $self->_buildSystem({
        'EPagesJBaseURL' => 'http://unstable-main.epages.com:8088/rs/'
    });

    my $SUT = DE_EPAGES::Shop::API::ShopUriBuilder->new($Shop, $System);
    my $PublicUri = $SUT->buildUri($PathTemplate, $hPathParams, $hQueyParams);
    my $PrivateUri = $SUT->buildPrivateUri($PathTemplate, $hPathParams, $hQueyParams);

    is($PublicUri, 'https://shop-domain.tld/rs/some/resource?color=ocean&search=ball', "$SubTestName - ($PublicUri) with query-params.");
    is($PrivateUri, 'http://unstable-main.epages.com:8088/rs/rs/some/resource?color=ocean&search=ball', "$SubTestName - ($PrivateUri) with query-params.");
}

sub test_withoutQueryParams {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->_buildShop({
        'RESTAPIURL' => 'https://shop-domain.tld/',
    });
    my $PathTemplate = '/rs/some/resource';
    my $hPathParams = {};
    my $hQueyParams = undef;
    my $System = $self->_buildSystem({
        'EPagesJBaseURL' => 'http://unstable-main.epages.com:8088/rs/'
    });

    my $SUT = DE_EPAGES::Shop::API::ShopUriBuilder->new($Shop, $System);
    my $PublicUri = $SUT->buildUri($PathTemplate, $hPathParams, $hQueyParams);
    my $PrivateUri = $SUT->buildPrivateUri($PathTemplate, $hPathParams, $hQueyParams);

    is($PublicUri, 'https://shop-domain.tld/rs/some/resource', "$SubTestName - ($PublicUri) without query-params.");
    is($PrivateUri, 'http://unstable-main.epages.com:8088/rs/rs/some/resource', "$SubTestName - ($PrivateUri) without query-params.");
}

sub test_withBrokenQueryParams {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->_buildShop({
        'RESTAPIURL' => 'https://shop-domain.tld/',
    });
    my $PathTemplate = '/rs/some/resource';
    my $hPathParams = {};
    my $hQueyParams = {'key' => undef};
    my $System = $self->_buildSystem({
        'EPagesJBaseURL' => 'http://unstable-main.epages.com:8088/rs/'
    });

    my $SUT = DE_EPAGES::Shop::API::ShopUriBuilder->new($Shop, $System);
    my $PublicUri = $SUT->buildUri($PathTemplate, $hPathParams, $hQueyParams);
    my $PrivateUri = $SUT->buildPrivateUri($PathTemplate, $hPathParams, $hQueyParams);

    is($PublicUri, 'https://shop-domain.tld/rs/some/resource?key=', "$SubTestName - ($PublicUri) with broken query-params.");
    is($PrivateUri, 'http://unstable-main.epages.com:8088/rs/rs/some/resource?key=', "$SubTestName - ($PrivateUri) with broken query-params.");
}

sub _buildShop {
    my $self = shift;
    my ($hAttributes) = @_;
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
}

sub _buildSystem {
    my $self = shift;
    my ($hAttributes) = @_;
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Site', $hAttributes);
}

__PACKAGE__->runScript();